import { GsStorageFormat } from "../ps2/gs_constants.js";
import Blob from "cross-blob";
export interface ImageLocation {
    width: number;
    height: number;
    format: GsStorageFormat;
    /**
     * Location of the image within GS memory, in units of 256 bytes
     */
    location: number;
    is_clut: boolean;
}
export declare class ImageChunk {
    width: number;
    height: number;
    locations: ImageLocation[];
    data: Uint8Array;
    base: number;
    /**
     *
     * @param width Width in PSMCT32 format. Width is doubled in PSMT8 and PSMT4, and halved in CLUTs.
     * @param height Height in PSMCT32 format. Height is doubled in PSMT8 and in CLUTS, and quadrupled in PSMT4.
     * @param data Image data in PSMCT32 format
     * @param locations A list of locations within the image file
     * @param base The default base address within the GS memory to write the image data to. Usually 0, but is set to 0x2a00 in fullscreen images and in the frontend data icon file.
     */
    constructor(width?: number, height?: number, locations?: ImageLocation[], data?: Uint8Array, base?: number);
    /**
     *
     * @param where
     * @param swap_red_blue Only relevant in PSMCT32 - swaps between RGB and BGR
     * @param fixup_alpha Only relevant in PSMCT32 - Remap alpha from 0x00-0x80 to 0x00-0xFF
     * @returns
     */
    export_data(where: ImageLocation, swap_red_blue?: boolean, fixup_alpha?: boolean): Uint8Array;
    /**
     *
     * @param where
     * @param input
     * @param swap_red_blue Only relevant in PSMCT32 - swaps between RGB and BGR
     * @param fixup_alpha Only relevant in PSMCT32 - Remap alpha from 0x00-0xFF to 0x00-0x80
     * @returns
     */
    import_data(where: ImageLocation, input: Uint8Array, swap_red_blue?: boolean, fixup_alpha?: boolean): void;
    export_indexed_data(where: ImageLocation, clut: ImageLocation, swap_red_blue?: boolean, fixup_alpha?: boolean): Uint8Array;
    import_indexed_data(where: ImageLocation, input: Uint8Array, clut: ImageLocation, swap_red_blue?: boolean, fixup_alpha?: boolean, premultiply?: boolean): void;
    static from_blob(blob: Blob): Promise<ImageChunk>;
    find_location(location_number: number): ImageLocation | undefined;
    to_blob(): Blob;
}
