import Blob from "cross-blob";
export declare enum ChunkType {
    EOF = 0,
    Materials = 2,
    WorldModel = 3,
    Image = 4,
    Collision = 5,
    ModelList = 8,
    FineCollision = 11,
    DynamicModel = 12,
    Model = 13,
    CharacterAssets = 18,
    Skeleton = 18,
    WorldGrid = 19,
    InstancedModels = 20,
    DynamicObjects = 29,
    Lights = 30,
    Header = 31,
    AssetGroup = 32,
    ShadowModel = 33,
    ZoneVis = 35,
    LevelDLL = 37,
    Fog = 41,
    Table = 42,
    SectorMap = 43,
    DialogueTable = 1000,
    WorldInfo = 1001
}
export declare class ChunkFile {
    chunks: Chunk[];
    constructor(chunks: Chunk[]);
    static from_blob(blob: Blob): Promise<ChunkFile>;
    get_chunk_of_type(type: number, index?: number): Chunk;
    get_chunks_of_type(type: number): Chunk[];
    get_chunk_by_id(type: number, id: number, index?: number): Chunk;
    delete_chunk(chunk: Chunk): void;
    copy(): ChunkFile;
    to_blob(): Blob;
    /**
     * Rebuilds the collision world grid. Use after modifying collision. See {@link GridChunk.rebuild}
     */
    rebuild_grid(): Promise<void>;
    original_size: number | undefined;
}
export declare class Chunk {
    contents: Blob;
    type: number;
    id: number;
    padding_bytes: number;
    offset: number;
    constructor(contents: Blob, type: number, id?: number, padding_bytes?: number);
    static from_blob(blob: Blob): Promise<Chunk>;
    get_alignment(): number;
    to_blob(offset?: number): Blob;
    copy(): Chunk;
}
