export declare enum VifCodeType {
    NOP = 0,
    STCYCL = 1,
    OFFSET = 2,
    BASE = 3,
    ITOP = 4,
    STMOD = 5,
    MSKPATH3 = 6,
    MARK = 7,
    FLUSHE = 16,
    FLUSH = 17,
    FLUSHA = 19,
    MSCAL = 20,
    MSCNT = 23,
    MSCALF = 21,
    STMASK = 32,
    STROW = 48,
    STCOL = 49,
    MPG = 74,
    DIRECT = 80,
    DIRECTHL = 81,
    UNPACK = 96
}
interface VifCodeBase {
    type: VifCodeType;
    interrupt: boolean;
}
export interface VifCodeSimple extends VifCodeBase {
    type: Exclude<VifCodeType, VifCodeType.STMASK | VifCodeType.STROW | VifCodeType.STCOL | VifCodeType.MPG | VifCodeType.DIRECT | VifCodeType.DIRECTHL | VifCodeType.UNPACK>;
    num: number;
    immediate: number;
}
export interface VifCodeStMask extends VifCodeBase {
    type: VifCodeType.STMASK;
    mask: number;
}
export interface VifCodeStRowCol extends VifCodeBase {
    type: VifCodeType.STROW | VifCodeType.STCOL;
    filling_data: [number, number, number, number];
}
export interface VifCodeDirect extends VifCodeBase {
    type: VifCodeType.DIRECT | VifCodeType.DIRECTHL;
    data: ArrayBuffer;
}
export interface VifCodeMpg extends VifCodeBase {
    type: VifCodeType.MPG;
    loadaddr: number;
    data: ArrayBuffer;
}
export interface VifCodeUnpack extends VifCodeBase {
    type: VifCodeType.UNPACK;
    masked: boolean;
    location: number;
    num: number;
    use_tops: boolean;
    unsigned: boolean;
    vn: number;
    vl: number;
    data: ArrayBuffer;
}
export declare type VifCode = VifCodeSimple | VifCodeStMask | VifCodeStRowCol | VifCodeDirect | VifCodeMpg | VifCodeUnpack;
export declare function read_vif_code(dv: DataView, pointer?: number, length?: number): VifCode[];
export declare function write_vif_code(vif_code: VifCode[]): ArrayBuffer;
export {};
